package bensley.FutureRadio;

// Please sent me ideas, updates and improvements;
// jwbensley@gmaill.com
// www.futureradio.co.uk

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.media.AudioManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.Uri;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

public class FutureRadio extends Activity {

	// Interface variables and objects
	static Button BTN_LISTEN32;
	static Button BTN_LISTEN64;
	public static TextView TXT_DURATION;
	
	// Menu items
	public static final int INT_MENU_MSG = 0;
	public static final int INT_MENU_CAM = 1;
	public static final int INT_MENU_SMS = 2;
	public static final int INT_MENU_WWW = 3;
	public static final int INT_MENU_FB = 4;
	public static final int INT_MENU_T = 5;
	
	// Messenger counters -
	// These are declared here because the studio messenger where these are used is a separate activity
	// Pressing back and then starting the studio messenger again will reinitialise the variables so they won't do their job
	public static long LNG_LAST_POST = 0;
	public static long LNG_THIS_POST;
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        
        // Interface with the UI elements and set onClick listener methods for the buttons
        BTN_LISTEN32 = (Button)this.findViewById(R.id.btnListen32);
        BTN_LISTEN32.setOnClickListener(new Button.OnClickListener() {public void onClick(View v) {click32();} });
        BTN_LISTEN64 = (Button)this.findViewById(R.id.btnListen64);
        BTN_LISTEN64.setOnClickListener(new Button.OnClickListener() {public void onClick(View v) {click64();} });
        TXT_DURATION = (TextView)this.findViewById(R.id.txtDuration);
        
        // Lock the context of the volume buttons
        setVolumeControlStream(AudioManager.STREAM_MUSIC);
        
    	if(StreamService.INT_PLAY_STATE==0){ // When calling the onCreate method, check the current streaming state in case it is being re-called 
    		// because the activity was sent to the background meaning the stream is already running (so the UI just needs updating to match this),
    		// otherwise we would leave the UI in the default state because this is the first call
    		BTN_LISTEN32.setText(R.string.strListen32);
    		BTN_LISTEN64.setText(R.string.strListen64);
    	}else if(StreamService.INT_PLAY_STATE==1){
    		BTN_LISTEN32.setText(R.string.strStop);
    		BTN_LISTEN64.setText(R.string.strListen64);
    		BTN_LISTEN64.setVisibility(4);
    	}else if(StreamService.INT_PLAY_STATE==2){
    		BTN_LISTEN32.setText(R.string.strListen32);
    		BTN_LISTEN32.setVisibility(4);
    		BTN_LISTEN64.setText(R.string.strStop);
    	}
    	
    	// Include these libs to be able to spit out the debug info below;
    	// We could add in an anonymous submission feature to gather details
    	// below like android version and phone make/model for screen sizes,
    	// to help test and improve the app?
    	//
    	//import android.app.Dialog;
    	//import android.content.ContentResolver;
    	//import android.content.res.Configuration;
    	//import android.telephony.SmsManager;
    	//import android.view.KeyEvent;
    	//
    	//Toast.makeText(this, "OS Version: " + System.getProperty("os.version")+ "(" + android.os.Build.VERSION.INCREMENTAL + ")", Toast.LENGTH_LONG).show();
    	//Log.d("OS Version: ", System.getProperty("os.version")+ "(" + android.os.Build.VERSION.INCREMENTAL + ")");
    	//Toast.makeText(this, "OS API Level: " + android.os.Build.VERSION.SDK, Toast.LENGTH_LONG).show();
    	//Log.d("OS API Level: ", android.os.Build.VERSION.SDK);
    	//Toast.makeText(this, "Device: " + android.os.Build.DEVICE, Toast.LENGTH_LONG).show();
    	//Log.d("Device: ", android.os.Build.DEVICE);
    	//Toast.makeText(this, "Model (and Product): " + android.os.Build.MODEL + " ("+ android.os.Build.PRODUCT + ")", Toast.LENGTH_LONG).show();
    	//Log.d("Model (and Product): ", android.os.Build.MODEL + " ("+ android.os.Build.PRODUCT + ")");
    	//Toast.makeText(this, "OS Name: " + System.getProperty("os.name"), Toast.LENGTH_LONG).show();
    	//Log.d("OS Name: ", System.getProperty("os.name"));
    	//Toast.makeText(this, "OS Arch: " + System.getProperty("os.arch"), Toast.LENGTH_LONG).show();
    	//Log.d("OS Arch: ", System.getProperty("os.arch"));
    	  	
    	if(isOnline()==false){ //Give the user a Toast prompt that no network connection has been detected
    		Toast.makeText(this, "No network connection has been detected!", Toast.LENGTH_SHORT).show();
    	}
    	
    }

    public boolean onCreateOptionsMenu(Menu menu){ //Draw the menu here in code rather than XML
    	// Possibly should be done in XML but if its different on every activity I don't think there's much difference
    	menu.add(0,INT_MENU_MSG,0,getString(R.string.strMessenger));
    	menu.add(0,INT_MENU_CAM,0,getString(R.string.strWebcam));
    	menu.add(0,INT_MENU_SMS,0,getString(R.string.strSMS));
    	menu.add(0,INT_MENU_WWW,0,getString(R.string.strWebsite));
    	menu.add(0,INT_MENU_FB,0,getString(R.string.strFacebook));
    	menu.add(0,INT_MENU_T,0,getString(R.string.strTwitter));
    	return true;
    }
    
    public boolean onOptionsItemSelected (MenuItem item){ // Menu item intents for each menu item index
    	switch (item.getItemId()){
    		case INT_MENU_MSG: // Studio (or instant) messenger selected, launch the activity
    			Intent intMessenger = new Intent(FutureRadio.this, StudioMessenger.class);
        		FutureRadio.this.startActivity(intMessenger);
    		return true;
    		case INT_MENU_CAM: //Web cam selected, launch the web cam activity to refresh the image
    			Intent intCam = new Intent(FutureRadio.this, WebCam.class);
        		FutureRadio.this.startActivity(intCam);
    		return true;
    		case INT_MENU_SMS: //Text studio selected, call the method to first check for SMS capabilities on this device,
    			// an intent is launched from there
    			trySMS();
    		return true;
    		case INT_MENU_WWW: //Fire the browser off to the main web site
    			Intent intWWW = new Intent(Intent.ACTION_VIEW);
    			intWWW.setData(Uri.parse("http://www.futureradio.co.uk/"));
        		FutureRadio.this.startActivity(intWWW);
    		return true;
    		case INT_MENU_FB: //Fire the browser off to the facebook page
    			Intent intFB = new Intent(Intent.ACTION_VIEW);
    			intFB.setData(Uri.parse("http://www.facebook.com/futureradio"));
        		FutureRadio.this.startActivity(intFB);
        	return true;
    		case INT_MENU_T: //Fire the browser off to the twitter page
    			Intent intT = new Intent(Intent.ACTION_VIEW);
    			intT.setData(Uri.parse("http://twitter.com/futureradio"));
        		FutureRadio.this.startActivity(intT);
    		return true;
    	}
    	return false;
    }
    
    public boolean isOnline() { //Method to check if a network connection is detected, returns true if so;
        ConnectivityManager connectivityManager = (ConnectivityManager)getApplicationContext().getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo networkInfo = connectivityManager.getActiveNetworkInfo();
        boolean connected = networkInfo != null && networkInfo.isAvailable() && networkInfo.isConnected();
        return connected;
    }
    
    public void click32() { //Method called when clicking the "listen 32k" button:
    	if(StreamService.INT_PLAY_STATE==1){ //1 means the 32k stream is playing...
    		stopService(new Intent(this, StreamService.class)); //So stop it
    		//BTN_LISTEN32.setText(R.string.strListen32); // Update UI
    		//BTN_LISTEN64.setVisibility(0);
    	}else if(StreamService.INT_PLAY_STATE==0){ // 0 means no stream is playing, so we want to start it
        	if(isOnline()==true){ // First check we actually have network connectivity
        		listen32(); // Yes we do, so call the method to begin listen to the LQ stream
        	}else{ // If returned false, it could just be a miss-match between the code and device
        		// so prompt the user 'yes' or 'no' in case they DO have connectivity
        		AlertDialog.Builder connectionAlertBuilder = new AlertDialog.Builder(this); //A separate AlertDialog builder has been created for each stream, because on clicking 'yes' a different stream is started
        		connectionAlertBuilder.setMessage("No network connection has been detected on this device, are you sure you want to try and listen?");
        		connectionAlertBuilder.setPositiveButton("Yes", listen32ClickListener);
        	    connectionAlertBuilder.setNegativeButton("No", listen32ClickListener);
        	    connectionAlertBuilder.show();
        	}
    	}
    }
    
    public void click64() {
    	if(StreamService.INT_PLAY_STATE==2){ // 2 means the 64k stream is playing
    		stopService(new Intent(this, StreamService.class)); //So stop it
    		//BTN_LISTEN64.setText(R.string.strListen64); //Update UI
    		//BTN_LISTEN32.setVisibility(0);
    	}else if(StreamService.INT_PLAY_STATE==0){ // 0 means no stream is playing
        	if(isOnline()==true){ // First check we actually have network connectivity
        		listen64(); // Yes we do, so call the method to begin listen to the HQ stream
        	}else{ // If returned false, it could just be a miss-match between the code and device
        		// so prompt the user 'yes' or 'no' in case they DO have connectivity
        		AlertDialog.Builder connectionAlertBuilder = new AlertDialog.Builder(this); //A separate AlertDialog builder has been created for each stream, because on clicking 'yes' a different stream is started
        		connectionAlertBuilder.setMessage("No network connection has been detected on this device, are you sure you want to try and listen?");
        		connectionAlertBuilder.setPositiveButton("Yes", listen64ClickListener);
        	    connectionAlertBuilder.setNegativeButton("No", listen64ClickListener);
        	    connectionAlertBuilder.show();
        	}
    	}
    }
    
    public void listen32(){ // Method to tune into the LQ stream
    	StreamService.STR_URL = getString(R.string.str32URL); // Set the stream URL to the low quality stream
		startService(new Intent(this, StreamService.class)); // Start the streaming service
		//BTN_LISTEN32.setText(R.string.strStop); // Update UI
		//BTN_LISTEN64.setVisibility(4);
    }
    
    public void listen64(){ // Method to tune into the HQ stream
    	StreamService.STR_URL = getString(R.string.str64URL);; // Set the stream URL to the high quality stream
		startService(new Intent(this, StreamService.class)); // Start the streaming service
		//BTN_LISTEN64.setText(R.string.strStop); // Update UI
		//BTN_LISTEN32.setVisibility(4);
    }
    
    DialogInterface.OnClickListener listen32ClickListener = new DialogInterface.OnClickListener() {
        @Override
        public void onClick(DialogInterface dialog, int which) {
            if(which==DialogInterface.BUTTON_POSITIVE){ //"Yes" button clicked
            	listen32(); // Call the method to tune into the LQ stream
            }
        }
    };
    
    DialogInterface.OnClickListener listen64ClickListener = new DialogInterface.OnClickListener() {
        @Override
        public void onClick(DialogInterface dialog, int which) {
            if(which==DialogInterface.BUTTON_POSITIVE){ //"Yes" button clicked
            	listen64(); // Call the method to tune into the HQ stream
            }
        }
    };
   
    public void trySMS() { // This method checks the package manager if this device has the TelephonyManager features (without which we assume there are no SMS capabilities)
    	if(this.getPackageManager().hasSystemFeature(PackageManager.FEATURE_TELEPHONY)){
    	    AlertDialog.Builder smsAlertBuilder = new AlertDialog.Builder(this);
    	    smsAlertBuilder.setMessage("No SMS capabilities have been detected on this device, are you sure you want to try and SMS the studio?");
    	    smsAlertBuilder.setPositiveButton("Yes", smsAlertClickListener);
    	    smsAlertBuilder.setNegativeButton("No", smsAlertClickListener);
    	    smsAlertBuilder.show();
    	}else{
    		launchSMS();
    	}
    }
    
    DialogInterface.OnClickListener smsAlertClickListener = new DialogInterface.OnClickListener() {
        @Override
        public void onClick(DialogInterface dialog, int which) {
        	if(which==DialogInterface.BUTTON_POSITIVE){ // "Yes" button clicked
        		launchSMS();
            }
        }
    };
    
    public void launchSMS() {
    	Intent intSMS = new Intent(Intent.ACTION_VIEW); // Create the new intent to launch the SMS activity
		intSMS.setData(Uri.parse("sms:")); // This URI is what defines this VIEW action as an SMS activity
		intSMS.putExtra("address", "60066"); // Pre-populate the SMS with the studio number and the required prefix 'future'
		intSMS.putExtra("sms_body", "future ");
		intSMS.setType("vnd.android-dir/mms-sms");
		FutureRadio.this.startActivity(intSMS);
    }

}