package bensley.FutureRadio;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.NameValuePair;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.media.AudioManager;
import android.os.AsyncTask;
import android.os.Bundle;
import android.text.Editable;
import android.text.InputFilter;
import android.text.TextWatcher;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

public class StudioMessenger extends Activity {
	
	// Interface variables and objects
	Button BTN_SEND;
	EditText TXT_NAME;
	EditText TXT_MESSAGE;
	TextView TXT_REMAINING;
	int INT_NAME_MAX_LEN = 35;
	int INT_MESSAGE_MAX_LEN = 355;
	InputFilter[] FLT_NAME = new InputFilter[1];
	InputFilter[] FLT_MESSAGE = new InputFilter[1];
	
	// Declare the HTTP response here because it is used inside the AsynTask inside a RunOnUiThead method
	HttpResponse response = null;
	
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.messenger);
        
        // Interface with the UI elements and set onClick listener methods for the buttons
        BTN_SEND = (Button)this.findViewById(R.id.btnSend);
        BTN_SEND.setOnClickListener(new Button.OnClickListener() {public void onClick(View v) {new postHTTPData().execute();} });
        TXT_NAME = (EditText)this.findViewById(R.id.txtName);
        FLT_NAME[0] = new InputFilter.LengthFilter(INT_NAME_MAX_LEN);
        TXT_NAME.setFilters(FLT_NAME);
    	TXT_MESSAGE = (EditText)this.findViewById(R.id.txtMessage);
    	FLT_MESSAGE[0] = new InputFilter.LengthFilter(INT_MESSAGE_MAX_LEN);
        TXT_MESSAGE.setFilters(FLT_MESSAGE);
        //TXT_MESSAGE.onKeyUp(keyCode, event)
    	TXT_REMAINING = (TextView)this.findViewById(R.id.txtRemaining);
    	
    	TXT_MESSAGE.addTextChangedListener(new TextWatcher() { //Here we implement a Change listener to capture key presses,
    		// something like the onKeyUp event for example could have been used but a TextChangedListener will capture EVERYTHING!!!
    		// So, we shall feed into a new TextWatcher however there are several inherit methods that must be implemented and but
    		// only the afterTextChanged method is wanted...Never mind :s
    		
    		public void onTextChanged(CharSequence cs, int s, int b, int c) {}
    		
			@Override
			public void afterTextChanged(Editable s) {
				TXT_REMAINING.setText("Remaining: " + String.valueOf(INT_MESSAGE_MAX_LEN-TXT_MESSAGE.length()));
				
			}

			@Override
			public void beforeTextChanged(CharSequence s, int start, int count, int after) {}
    	});
    	
    	// Lock the context of the volume buttons
    	setVolumeControlStream(AudioManager.STREAM_MUSIC);
    }
    
    // The main feature of this class runs in an AsyncTasks so we don't hog up the main UI thread and 
    // it allows the action to be interruptible
    private class postHTTPData extends AsyncTask<Void, Integer, Void>
    {
    
    	ProgressDialog PD_LOADING;
    	
    	protected void onPreExecute() {
    		// Make sure the user hasn't written too much or too little
    		if(TXT_MESSAGE.length()<=INT_MESSAGE_MAX_LEN && TXT_MESSAGE.length() > 0 && TXT_NAME.length() > 0 && TXT_NAME.length() <= INT_NAME_MAX_LEN){
    			// Make sure they user hasn't posted a message within the last 60 seconds
    			FutureRadio.LNG_THIS_POST = System.currentTimeMillis();
    		    if(FutureRadio.LNG_THIS_POST-FutureRadio.LNG_LAST_POST>60000){ // Limit posting of studio messages to once per minute
    		    	PD_LOADING = ProgressDialog.show(StudioMessenger.this, "Sending" , " Submitting, please wait ... ", true, true,
    		    			new DialogInterface.OnCancelListener(){
                        		@Override
                        		public void onCancel(DialogInterface dialog) {
                        			postHTTPData.this.cancel(true);
                        			finish();
                        		}
                    		}
    		    	);
    		    }else{ // LNG_LAST_POST else
    		    	Toast.makeText(StudioMessenger.this, "Please wait 60 seconds between each message", Toast.LENGTH_SHORT).show();
    		    	postHTTPData.this.cancel(true);
    		    }
    		}else{ // Message and name length else
    			Toast.makeText(StudioMessenger.this, "Please enter a name and message", Toast.LENGTH_SHORT).show();
    			postHTTPData.this.cancel(true);
    		}
    	}
    
    	@Override
    	protected Void doInBackground(Void... params) {
    		
	    	// Create a new HttpClient and HttpPost header
	    	HttpClient httpClient = new DefaultHttpClient();
	    	HttpPost httpPost = new HttpPost("http://futureradio.co.uk/studiomessenger");
	    	
	    	// Build up the PHP variable names and values
	    	List<NameValuePair> nameValuePairs = new ArrayList<NameValuePair>(2); 
		    nameValuePairs.add(new BasicNameValuePair("name", TXT_NAME.getText().toString())); 
		    nameValuePairs.add(new BasicNameValuePair("message", TXT_MESSAGE.getText().toString()));
	        
		    //Build the HTTP post
		    try {
		    	httpPost.setEntity(new UrlEncodedFormEntity(nameValuePairs));
		    } catch (UnsupportedEncodingException e) {
		    	Log.d("StudioMessenger Error: httpPost.setEntity()", " - UnsupportedEncodingException");
		    	e.printStackTrace();
		    }
	
		    // Try to execute the HTTP Post Request
		    try {
		    	response = httpClient.execute(httpPost);
		    } catch (ClientProtocolException e) {
		    	Log.d("StudioMessenger Error: httpClient.execute()", " - ClientProtocolException");
		    	e.printStackTrace();
		    } catch (IOException e) {
		    	Log.d("StudioMessenger Error: httpClient.execute()", " - IOException");
		    	e.printStackTrace();
		    }
		    
		    if(response != null){ //Did a HttpResponse even get returned at all?
		    	if(response.getStatusLine().getStatusCode() == HttpStatus.SC_OK){ //If we get the HTTP OK status code (200) back...
		    		FutureRadio.LNG_LAST_POST = FutureRadio.LNG_THIS_POST; // Reset the time in the last successful post
		    		runOnUiThread(new Runnable() {
		    			public void run() {
			    	    Toast.makeText(StudioMessenger.this, "Thank you, your message has been submitted", Toast.LENGTH_SHORT).show(); //Tell the user the good news!
			    	    TXT_NAME.setText(""); // Reset UI - Hence the runOnUiThread()
				    	TXT_MESSAGE.setText("");
			    	   }
		    		});
		    	}else{ // If there was any other HTTP response code, tell the user
			    	runOnUiThread(new Runnable() {
		    			public void run() {
		    				Toast.makeText(StudioMessenger.this, "Oh dear, your message has NOT been submitted!", Toast.LENGTH_LONG).show();
		    				Toast.makeText(StudioMessenger.this, "Tell some at Future, HTTP status " + response.getStatusLine().getStatusCode(), Toast.LENGTH_LONG).show();
				    	}
			    	});
		    	}
		    }else{ // No HttpResponse of any sort was returned
		    	runOnUiThread(new Runnable() {
	    			public void run() {
	    				Toast.makeText(StudioMessenger.this, "Oh dear, your message has NOT been submitted!", Toast.LENGTH_LONG).show();
	    				Toast.makeText(StudioMessenger.this, "I couldn't connect to Future :(", Toast.LENGTH_LONG).show();
	    			}
		    	});
	    	}
			    	
		    return null;
		    
    	} //doInBackground()
    
    	protected void onPostExecute(Void result) {
    		if(PD_LOADING.isShowing()){
    				PD_LOADING.dismiss();
    		}
    	}
    
    } //postHTTPData, AsyncTask
    
}
